﻿<#
.SYNOPSIS
	Determines whether database updates are required.

.DESCRIPTION
	Determines whether database updates are required. It performs the following checks:
		Detects whether the pool version is Lync Server 2013 or later
		Detects whether database mirroring is enabled
		Detects whether the primary and mirror SQL servers are online
		Detects whether the mirror server is principal for any databases
		Detects whether the local machine is a FE of the given pool
		Detects whether the CMS is on Lync Server 2013 or later
	And returns the following info:
		Returns list of local databases, their versions, and whether a database update is required (if the local server is a member of the pool)
		Returns list of backend databases, their versions, and whether a database update is required
		Returns list of CMS databases, their versions, and whether a database update is required

.NOTES
  Version      	   		: 1.1 - See changelog at http://www.ehloworld.com/2609
  Wish list						: Better error trapping
  Rights Required			: TBD
  Sched Task Required	: No
  Lync Server Version	: 2013 through January 2014 Cumulative Update
  Author(s)    				: Dave Howe
  										: Pat Richard
	Email/Blog/Twitter	: daveh@microsoft.com
												pat@innervation.com 	http://www.ehloworld.com @patrichard
	Dedicated Post			: http://www.ehloworld.com/2610
  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is provided AS IS
												without warranty of any kind. I disclaim all implied warranties including, without limitation, any implied
												warranties of merchantability or of fitness for a particular purpose. The entire risk arising out of the use
												or performance of the sample scripts and documentation remains with you. In no event shall I be liable for
												any damages whatsoever (including, without limitation, damages for loss of business profits, business
												interruption, loss of business information, or other pecuniary loss) arising out of the use of or inability
												to use the script or documentation.
	Acknowledgements 		:
	Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations					:
	Known issues				: Requires that Execution Policy be set to RemoteSigned or Unrestricted
											: script will abort if mirror sql server is principal for any lync database

.EXAMPLE
	.\Test-LyncDatabases.ps1

	Description
	-----------
	Runs the script. If there is only one pool in the environment, it will query that pool. If there are more pools, it will prompt for the pool FQDN to query.

.EXAMPLE
	.\Test-LyncDatabases.ps1 -PoolFqdn pool.contoso.com

	Description
	-----------
	Runs the script and queries the pool specified.
#>
#Requires -Version 2.0

[CmdletBinding(SupportsShouldProcess = $True)]
param(
	# FQDN of the Lync pool to query
	[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true)]
	[string] $PoolFqdn
)

#region functions

function Get-PoolData {
	[CmdletBinding(SupportsShouldProcess = $True)]
	param(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true)]
		$PoolFqdn
	)
	if (!(Get-CsComputer -Pool $PoolFqdn))		{
		Write-Host "`nerror: pool not found in topology`n`n" -ForegroundColor red
		break
	}

	if ((Get-CsService -PoolFqdn $PoolFqdn).Version[0] -lt 6) {
		Write-Host "`nerror: incorrect server version (Lync Server 2013 required)`n`n" -ForegroundColor red
		break
	}

	$FrontEnds = @((Get-CsPool $PoolFqdn).Computers | Sort-Object)

	Write-Host "`n`n        pool fqdn:" $PoolFqdn

	if ($FrontEnds.Count -eq 1)		{
		Write-Host "   front end fqdn:" $FrontEnds[0]
	}	else		{
		Write-Host "  front end fqdns:" $FrontEnds[0]
		for ($int = 1; $int -lt $FrontEnds.Count; $int ++)			{
			Write-Host "                  " $FrontEnds[$int]
		}
	}

	return $FrontEnds
} # end function Get-PoolData

function Get-SqlServerData {
	[CmdletBinding(SupportsShouldProcess = $True)]
	param(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true)]
		$PoolFqdn
	)
	$Services = @(Get-CsService -PoolFqdn $PoolFqdn)

	foreach ($Service in $Services)		{
		if ($Service.role -contains "UserServer")			{
			if ($Service.UserDatabase) {
				$PrimarySqlFqdn = $Service.UserDatabase.Split(":")[1]
			}
			if ($Service.MirrorUserDatabase) {
				$MirrorSqlFqdn = $Service.MirrorUserDatabase.Split(":")[1]
			}
			} elseif ($Service.role -contains "PersistentChat") {
				if ($Service.PersistentChatDatabase) {
					$PrimarySqlFqdn = $Service.PersistentChatDatabase.Split(":")[1]
				}
			if ($Service.MirrorPersistentChatDatabase) {
				$MirrorSqlFqdn = $Service.MirrorPersistentChatDatabase.Split(":")[1]
			}
		} elseif ($Service.role -contains "Registrar") {
			if ($Service.MonitoringDatabase) {
				$PrimarySqlFqdn = $Service.MonitoringDatabase.split(":")[1]
			}
			if ($Service.MirrorMonitoringDatabase) {
				$MirrorSqlFqdn = $Service.MirrorMonitoringDatabase.split(":")[1]
			}
		}
	}

	if ($PrimarySqlFqdn -ne $null)		{
		if(!(Test-Connection -cn $PrimarySqlFqdn -BufferSize 16 -Count 1 -ErrorAction SilentlyContinue -Quiet))			{
			Write-Host "`n`nerror: Primary sql server $PrimarySqlFqdn is down`n`n" -ForegroundColor red
			break
		}
	}

	if ($MirrorSqlFqdn -ne $null)		{
		if(!(Test-Connection -cn $MirrorSqlFqdn -BufferSize 16 -Count 1 -ErrorAction SilentlyContinue -Quiet))			{
			Write-Host "`n`nerror: Mirror sql server $MirrorSqlFqdn is down`n`n" -ForegroundColor red
			break
		}
	}

	if ($PrimarySqlFqdn -ne $null)		{
		Write-Host " primary sql fqdn:" $PrimarySqlFqdn -NoNewLine
		$Principal = @(Get-CsDatabaseMirrorState -PoolFqdn $PoolFqdn | Where-Object {$_.StateOnPrimary -match "Principal"} | Select-Object DatabaseName -ExpandProperty DatabaseName)
		if ($Principal)			{
			Write-Host " ($Principal)"
		}		else			{
			Write-Host
		}
	}

	if ($MirrorSqlFqdn -ne $null)		{
		Write-Host "  mirror sql fqdn:" $MirrorSqlFqdn -NoNewLine
		$Principal = @(Get-CsDatabaseMirrorState -PoolFqdn $PoolFqdn | Where-Object {$_.StateOnMirror -match "Principal"} | Select-Object DatabaseName -ExpandProperty DatabaseName)
		if ($Principal)			{
			Write-Host " ($Principal)`n"
			Write-Host "error: Failover databases to primary sql server`n`n" -ForegroundColor red
			break
		}		else			{
			Write-Host
		}
	}
	return $PrimarySqlFqdn, $MirrorSqlFqdn
} # end function Get-SQLServerData

function Get-DBTestResults {
	[CmdletBinding(SupportsShouldProcess = $True)]
	param(
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[ValidateSet("LocalDatabases", "BackendDatabases", "CmsDatabases")]
		[ValidateNotNullOrEmpty()]
		[string] $DBType,
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[string] $PoolFqdn,
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[string] $PrimarySqlFqdn
	)

	# // get database test results
	switch ($DBType) {
		LocalDatabases {
			if (((Get-CsPool $PoolFqdn).computers) -contains ((Get-CsComputer -Local).fqdn)){
				Write-Host "Local Databases: ($((Get-CsComputer -Local).fqdn))`n"
				$dbresults = @(Test-CsDatabase -LocalService | Where-Object {$_.SqlServerFqdn -match "(local)"})
			}
		}

		BackendDatabases {
			Write-Host "Backend Databases:`n"
			$dbresults = @(Test-CsDatabase -ConfiguredDatabases -SqlServerFqdn $PrimarySqlFqdn)
		}

		CmsDatabases {
			if (!(Get-CsService -CentralManagement).version[0] -lt 6) {
				Write-Host "Central Management Databases:`n"
				$dbresults = @(Test-CsDatabase -CentralManagementDatabase -SqlServerFqdn ((Get-CsManagementConnection).SqlServer) -SqlInstanceName ((Get-CsManagementConnection).SqlInstance))
			}
		}
	}

	for ($int=0; $int -lt $dbresults.count; $int ++) {
		$databasename     += @([string]$dbresults[$int].databasename)
		$expectedversion  += @([string]$dbresults[$int].expectedversion)
		$installedversion += @([string]$dbresults[$int].installedversion)

		if (($databasename -match "rtcdyn") -and ([string]$dbresults[$int].expectedversion -match "125.6.3") -and ([string]$dbresults[$int].installedversion -match "0.0.0")) {
			$updaterequired += @("no")
		} elseif (($databasename -match "xds") -and ([string]$dbresults[$int].installedversion -match "10.13.2"))			{
			$updaterequired += @("no")
		} elseif ([string]$dbresults[$int].expectedversion -match [string]$dbresults[$int].installedversion)			{
			$updaterequired += @("no")
		} elseif ([string]$dbresults[$int].expectedversion -notmatch [string]$dbresults[$int].installedversion)			{
			$updaterequired += @("yes")
		}
	}

	$DatabasesData = @()

	for ($int=0; $int -lt $dbresults.count; $int ++) {
		$ThisObject = New-Object PSObject
		Add-Member -InputObject $ThisObject -Type NoteProperty -Name DatabaseName -Value $DatabaseName[$int]
		Add-Member -InputObject $ThisObject -Type NoteProperty -Name ExpectedVersion -Value $ExpectedVersion[$int]
		Add-Member -InputObject $ThisObject -Type NoteProperty -Name InstalledVersion -Value $InstalledVersion[$int]
		Add-Member -InputObject $ThisObject -Type NoteProperty -Name UpdateRequired -Value $UpdateRequired[$int]
		$DatabasesData += $ThisObject
	}

	return $DatabasesData | Sort-Object DatabaseName | Format-Table -Autosize
} # end function Get-DBTestResults

function Set-ModuleStatus {
	[CmdletBinding(SupportsShouldProcess = $True)]
	param	(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage = "No module name specified!")]
		[ValidateNotNullOrEmpty()]
		[string] $name
	)
	PROCESS{
		# Executes once for each pipeline object
		# the $_ variable represents the current input object
		if (!(Get-Module -name "$name")) {
			if (Get-Module -ListAvailable | Where-Object Name -eq "$name") {
				Import-Module -Name "$name"
				# module was imported
				# return $true
			} else {
				# module was not available
				# return $false
			}
		} else {
			# Write-Output "$_ module already imported"
			# return $true
		}
	} # end PROCESS
} # end function Set-ModuleStatus

#end region functions

#region main
if ((! $PoolFqdn) -and ((Get-CsService -Registrar | Measure-Object).count -eq 1)){
	$PoolFqdn = (Get-CsService -Registrar).PoolFqdn
}
Write-Host

if (!$PoolFqdn){
	$PoolFqdn = Read-Host "server/pool FQDN"
}

Write-Host "`n`nChecking..." -NoNewLine

if ($PoolData = @(Get-PoolData $PoolFqdn))	{
	$FrontEnds = $PoolData
}

if ($SqlServerData = @(Get-SqlServerData -PoolFqdn $PoolFqdn)) {
	$PrimarySqlFqdn = $SqlServerData[0]
	$MirrorSqlFqdn = $SqlServerData[1]
}

Write-Host "`n`n"

Get-DBTestResults -DBType LocalDatabases -PoolFqdn $PoolFqdn
Get-DBTestResults -DBType BackendDatabases -PoolFqdn $PoolFqdn -PrimarySqlFqdn $PrimarySqlFqdn
Get-DBTestResults -DBType CmsDatabases

#end region main


# SIG # Begin signature block
# MIIQBQYJKoZIhvcNAQcCoIIP9jCCD/ICAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUebvjlRE+958zOiTa14qP1/xr
# V42ggg1KMIIGnzCCBYegAwIBAgIQCSu3nvOdPAJrDHL+XciEEzANBgkqhkiG9w0B
# AQUFADBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xMB4XDTEzMDgyODAwMDAwMFoXDTE0MDkwNDEy
# MDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBTdGVy
# bGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYDVQQD
# ExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEA8dXC+TSUZsMisaialKJLat7AnmOF0HE+U49duaBJp/N4IXhjUZM9sXRkR/X0
# KAbNauNpNrs3TLKNpO7/RUinzrzsg8IhtBVXSrK7c0HDqJ7vG181it2qQO1yB9uD
# UV8frjprnNcR1w0qW9wFqclKlkyOxgiCANVoT6qXO9IxPZT28h+tMfBCsU7sZALs
# 3kPRMCQ1ZywngVSqEQ9rBYV3jW4EgvPEOmuhtnwgVB1Alr2V1dVAtbNxw1A7fQ8w
# J6EQSNAPVJQq7304QKKgGaTXtA2TwAjO5+m9ZltD/s0wXWYs0V1r8pOJOo1r17l4
# z91OWLPP+yfxvlnm5F1IXZxCbQIDAQABo4IDOTCCAzUwHwYDVR0jBBgwFoAUe2jO
# KarAF75JeuHlP9an90WPNTIwHQYDVR0OBBYEFB9N9j4QNDPcSscWMAVDKCOcXS/G
# MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBzBgNVHR8EbDBq
# MDOgMaAvhi1odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vYXNzdXJlZC1jcy0yMDEx
# YS5jcmwwM6AxoC+GLWh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9hc3N1cmVkLWNz
# LTIwMTFhLmNybDCCAcQGA1UdIASCAbswggG3MIIBswYJYIZIAYb9bAMBMIIBpDA6
# BggrBgEFBQcCARYuaHR0cDovL3d3dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBv
# c2l0b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAg
# AG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBz
# AHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABo
# AGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABo
# AGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBu
# AHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAg
# AGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQBy
# AGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUAbgBjAGUALjCBggYIKwYBBQUHAQEE
# djB0MCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wTAYIKwYB
# BQUHMAKGQGh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3Vy
# ZWRJRENvZGVTaWduaW5nQ0EtMS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0B
# AQUFAAOCAQEABTcbxhPxSJP7AKrF23wXbgelEGzjpewmm0BnHO9/V+9K4UE/3i5Z
# e3bX7OvUb/55zCHT4pvZl8/i8tuvFnrWZZcjGljT3W9SIVOuKNdlItXC8dvP67Hz
# OMZvSbdI5MU6sjd3vyYKznTMHG1Mz/x+BMh6rEnGsIaY4r8r4fkNfJTyTkww8AWh
# I16Pno3dWgKhfploPODtMxczJKA15VDFhQqM3oHHMojasXo5uGb5jlGiDFkZ3mDa
# /+gLUsJUjAzF27ZIy/jRgp0eMkqlgdxGa+gZUK73Ue6aTP2LW9SL/1fLYl8QkwRs
# MWG1faGdureT3Oh0EHH6UhREB/SxLJ+xLTCCBqMwggWLoAMCAQICEA+oSQYV1wCg
# viF2/cXsbb0wDQYJKoZIhvcNAQEFBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoT
# DERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UE
# AxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTExMDIxMTEyMDAwMFoX
# DTI2MDIxMDEyMDAwMFowbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMTCCASIwDQYJKoZIhvcNAQEB
# BQADggEPADCCAQoCggEBAJx8+aCPCsqJS1OaPOwZIn8My/dIRNA/Im6aT/rO38bT
# JJH/qFKT53L48UaGlMWrF/R4f8t6vpAmHHxTL+WD57tqBSjMoBcRSxgg87e98tzL
# uIZARR9P+TmY0zvrb2mkXAEusWbpprjcBt6ujWL+RCeCqQPD/uYmC5NJceU4bU7+
# gFxnd7XVb2ZklGu7iElo2NH0fiHB5sUeyeCWuAmV+UuerswxvWpaQqfEBUd9YCvZ
# oV29+1aT7xv8cvnfPjL93SosMkbaXmO80LjLTBA1/FBfrENEfP6ERFC0jCo9dAz0
# eotyS+BWtRO2Y+k/Tkkj5wYW8CWrAfgoQebH1GQ7XasCAwEAAaOCA0MwggM/MA4G
# A1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzCCAcMGA1UdIASCAbow
# ggG2MIIBsgYIYIZIAYb9bAMwggGkMDoGCCsGAQUFBwIBFi5odHRwOi8vd3d3LmRp
# Z2ljZXJ0LmNvbS9zc2wtY3BzLXJlcG9zaXRvcnkuaHRtMIIBZAYIKwYBBQUHAgIw
# ggFWHoIBUgBBAG4AeQAgAHUAcwBlACAAbwBmACAAdABoAGkAcwAgAEMAZQByAHQA
# aQBmAGkAYwBhAHQAZQAgAGMAbwBuAHMAdABpAHQAdQB0AGUAcwAgAGEAYwBjAGUA
# cAB0AGEAbgBjAGUAIABvAGYAIAB0AGgAZQAgAEQAaQBnAGkAQwBlAHIAdAAgAEMA
# UAAvAEMAUABTACAAYQBuAGQAIAB0AGgAZQAgAFIAZQBsAHkAaQBuAGcAIABQAGEA
# cgB0AHkAIABBAGcAcgBlAGUAbQBlAG4AdAAgAHcAaABpAGMAaAAgAGwAaQBtAGkA
# dAAgAGwAaQBhAGIAaQBsAGkAdAB5ACAAYQBuAGQAIABhAHIAZQAgAGkAbgBjAG8A
# cgBwAG8AcgBhAHQAZQBkACAAaABlAHIAZQBpAG4AIABiAHkAIAByAGUAZgBlAHIA
# ZQBuAGMAZQAuMBIGA1UdEwEB/wQIMAYBAf8CAQAweQYIKwYBBQUHAQEEbTBrMCQG
# CCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQwYIKwYBBQUHMAKG
# N2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJv
# b3RDQS5jcnQwgYEGA1UdHwR6MHgwOqA4oDaGNGh0dHA6Ly9jcmwzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwOqA4oDaGNGh0dHA6Ly9j
# cmw0LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwHQYD
# VR0OBBYEFHtozimqwBe+SXrh5T/Wp/dFjzUyMB8GA1UdIwQYMBaAFEXroq/0ksuC
# MS1Ri6enIZ3zbcgPMA0GCSqGSIb3DQEBBQUAA4IBAQB7ch1k/4jIOsG36eepxIe7
# 25SS15BZM/orh96oW4AlPxOPm4MbfEPE5ozfOT7DFeyw2jshJXskwXJduEeRgRNG
# +pw/alE43rQly/Cr38UoAVR5EEYk0TgPJqFhkE26vSjmP/HEqpv22jVTT8nyPdNs
# 3CPtqqBNZwnzOoA9PPs2TJDndqTd8jq/VjUvokxl6ODU2tHHyJFqLSNPNzsZlBjU
# 1ZwQPNWxHBn/j8hrm574rpyZlnjRzZxRFVtCJnJajQpKI5JA6IbeIsKTOtSbaKbf
# KX8GuTwOvZ/EhpyCR0JxMoYJmXIJeUudcWn1Qf9/OXdk8YSNvosesn1oo6WQsQz/
# MYICJTCCAiECAQEwgYMwbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMQIQCSu3nvOdPAJrDHL+XciE
# EzAJBgUrDgMCGgUAoHgwGAYKKwYBBAGCNwIBDDEKMAigAoAAoQKAADAZBgkqhkiG
# 9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIB
# FTAjBgkqhkiG9w0BCQQxFgQU4A0wEfjPGb26wGVVEENaOPiHFfUwDQYJKoZIhvcN
# AQEBBQAEggEA4uUWLNSq1/P0/r6m3I697kdzDdOmETwrYZ9hSOi7tAb6w0E+u4Oi
# lTO+BUjaiXylfCuoV8PE4YcrfnHVN46+VYE/IJKZLdmf4NUUx906WlEqLaphU7Dx
# GuHJVVJQr7BRXX+P/W9bpXZHUkJFctsb4+pInczSQm1LL28Qjkuttk21SjDQ42jB
# fInqjB4ToVsVwbcQkKIQlPKmri393e2vR9yHf5qTtzjQ1aFkYPi2+t8uSMTluJDF
# nHf18BfVDH0TfaWNrCZtEeRXyTcnIT3tkA68GirfdcVF20I8PHwFn1jc+iraPhRo
# 9bd8rIZKloXmjEh0U9/qWQEIQGBswqgSAA==
# SIG # End signature block
